﻿using Fleck;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

/*
 *         Progetto: Trasmissione di dati di temperatura mediante Websocket
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 11/12/2015
 *  
 *      Descrizione: Simulatore di sensore di temperatura: genera valori di temperatura a intervalli di 
 *                   tempo predefiniti; crea connessioni durature con i client mediante websocket;
 *                   invia le temperature ai client connessi; consente l'impostazione dell'unità di misura.
 *                   
 */
namespace ServerTemperatura
{
    class Program
    {
        static Dictionary<IWebSocketConnection, char> connessioni = new Dictionary<IWebSocketConnection, char>();
        static double temperatura = 20.0;

        static void Main(string[] args)
        {
            double delta = +0.1;

            WebSocketServer server = new WebSocketServer("ws://0.0.0.0");

            server.Start(socket =>
            {
                socket.OnOpen = () => GestisciApertura(socket);
                socket.OnClose = () => GestisciChiusura(socket);
                socket.OnMessage = message => GestisciMessaggio(socket, message);
            });

            while (true)
            {
                foreach (var coppia in connessioni)
                {
                    var socket = coppia.Key;
                    var um = coppia.Value;

                    string t = Misura(temperatura, um);
                    socket.Send(t);
                }

                Thread.Sleep(2000);     // Attendere due secondi prima di elaborare una nuova temperatura

                temperatura += delta;

                if ((temperatura >= 25.0 && delta > 0) || (temperatura <= 18.0 && delta < 0))
                    delta = -delta;
            }
            

        }

        private static void GestisciMessaggio(IWebSocketConnection sck, string messaggio)
        {
            string t;
            messaggio = messaggio.Trim().ToUpper();

            if (messaggio.Equals("CELSIUS")) {
                t = Misura(temperatura, 'C');
                connessioni[sck] = 'C';
            }
            else if (messaggio.Equals("FAHRENHEIT")) {
                t = Misura(temperatura, 'F');
                connessioni[sck] = 'F';
            }
            else
                t = "ERR";

            sck.Send(t);
        }

        private static void GestisciApertura(IWebSocketConnection sck)
        {
            connessioni.Add(sck, 'C');
            Console.WriteLine(String.Format("Aggiunto: {0}:{1}",
                sck.ConnectionInfo.ClientIpAddress,
                sck.ConnectionInfo.ClientPort)
            );
        }

        private static void GestisciChiusura(IWebSocketConnection sck)
        {
            connessioni.Remove(sck);
            Console.WriteLine(String.Format("Rimosso: {0}:{1}", 
                sck.ConnectionInfo.ClientIpAddress, 
                sck.ConnectionInfo.ClientPort)
            );
        }

        private static string Misura(double tc, char um) {
            double tf = tc * 1.8 + 32.0;

            return String.Format("{0:N1} °{1}", (um == 'F' ? tf : tc), um);
        }
    }
}
